@@ -448,6 +448,11 @@ public class ObjectOutputStream extends OutputStream
         realOutput.writeByte(flags);
 
         ObjectStreamField[] fields = osc.fields;
+
+	if (fields == ObjectStreamClass.INVALID_FIELDS)
+	  throw new InvalidClassException
+		  (osc.getName(), "serialPersistentFields is invalid");
+
         realOutput.writeShort(fields.length);
 
         ObjectStreamField field;
