@@ -243,7 +243,8 @@ public class WeakHashMap<K,V> extends AbstractMap<K,V>
           // This method will get inlined.
           cleanQueue();
           if (knownMod != modCount)
-            throw new ConcurrentModificationException();
+            throw new ConcurrentModificationException(knownMod + " != "
+                                                      + modCount);
         }
 
         /**
@@ -476,7 +477,7 @@ public class WeakHashMap<K,V> extends AbstractMap<K,V>
         if (o instanceof Map.Entry)
           {
             Map.Entry e = (Map.Entry) o;
-            return key.equals(e.getKey())
+            return WeakHashMap.equals(getKey(), e.getKey())
               && WeakHashMap.equals(value, e.getValue());
           }
         return false;
@@ -484,7 +485,7 @@ public class WeakHashMap<K,V> extends AbstractMap<K,V>
 
       public String toString()
       {
-        return key + "=" + value;
+        return getKey() + "=" + value;
       }
     }
 
@@ -658,7 +659,7 @@ public class WeakHashMap<K,V> extends AbstractMap<K,V>
     while (bucket != null)
       {
         WeakBucket.WeakEntry entry = bucket.getEntry();
-        if (entry != null && key.equals(entry.key))
+        if (entry != null && equals(key, entry.key))
           return entry;
 
         bucket = bucket.next;
@@ -700,21 +701,20 @@ public class WeakHashMap<K,V> extends AbstractMap<K,V>
     // bucket may be enqueued later by the garbage collection, and
     // internalRemove will be called a second time.
     bucket.slot = -1;
-    if (buckets[slot] == bucket)
-      buckets[slot] = bucket.next;
-    else
+
+    WeakBucket prev = null;
+    WeakBucket next = buckets[slot];
+    while (next != bucket)
       {
-        WeakBucket prev = buckets[slot];
-        /* This may throw a NullPointerException.  It shouldn't but if
-         * a race condition occurred (two threads removing the same
-         * bucket at the same time) it may happen.  <br>
-         * But with race condition many much worse things may happen
-         * anyway.
-         */
-        while (prev.next != bucket)
-          prev = prev.next;
-        prev.next = bucket.next;
+         if (next == null) throw new InternalError("WeakHashMap in incosistent state");
+         prev = next; 
+         next = prev.next;
       }
+    if (prev == null)
+      buckets[slot] = bucket.next;
+    else 
+      prev.next = bucket.next;
+
     size--;
   }
 
