@@ -344,7 +344,7 @@ public class SwingUtilities
    * @see #getAncestorOfClass
    * @see #windowForComponent
    */
-  public static Container getAncestorOfClass(Class c, Component comp)
+  public static Container getAncestorOfClass(Class<?> c, Component comp)
   {
     while (comp != null && (! c.isInstance(comp)))
       comp = comp.getParent();
@@ -1138,14 +1138,12 @@ public class SwingUtilities
     else
       {
         ActionMap parent = child.getParent();
-        while(parent != null)
+        while (parent != null && !(parent instanceof ActionMapUIResource))
           {
             child = parent;
             parent = child.getParent();
           }
-
-        if (child != null)
-          child.setParent(uiActionMap);
+        child.setParent(uiActionMap);
       }
   }
 
@@ -1181,11 +1179,13 @@ public class SwingUtilities
       component.setInputMap(condition, uiInputMap);
     else
       {
-        while(child.getParent() != null
-              && !(child.getParent() instanceof InputMapUIResource))
-          child = child.getParent();
-        if (child != null)
-          child.setParent(uiInputMap);
+        InputMap parent = child.getParent();
+        while (parent != null && !(parent instanceof InputMapUIResource))
+          {
+            child = parent;
+            parent = parent.getParent();
+          }
+        child.setParent(uiInputMap);
       }
   }
 
@@ -1395,4 +1395,30 @@ public class SwingUtilities
     else
       return null;
   }
+
+  /**
+   * Processes key bindings for the component that is associated with the 
+   * key event. Note that this method does not make sense for
+   * JComponent-derived components, except when
+   * {@link JComponent#processKeyEvent(KeyEvent)} is overridden and super is
+   * not called.
+   *
+   * This method searches through the component hierarchy of the component's
+   * top-level container to find a <code>JComponent</code> that has a binding
+   * for the key event in the WHEN_IN_FOCUSED_WINDOW scope.
+   *
+   * @param ev the key event
+   *
+   * @return <code>true</code> if a binding has been found and processed,
+   *         <code>false</code> otherwise
+   *
+   * @since 1.4
+   */
+  public static boolean processKeyBindings(KeyEvent ev)
+  {
+    Component c = ev.getComponent();
+    KeyStroke s = KeyStroke.getKeyStrokeForEvent(ev);
+    KeyboardManager km = KeyboardManager.getManager();
+    return km.processKeyStroke(c, s, ev);
+  }
 }
