@@ -54,6 +54,7 @@ import java.util.ResourceBundle;
 
 import javax.swing.border.Border;
 import javax.swing.plaf.ComponentUI;
+import javax.swing.plaf.InputMapUIResource;
 
 /**
  * UIDefaults is a database where all settings and interface bindings are
@@ -62,7 +63,7 @@ import javax.swing.plaf.ComponentUI;
  *
  * @author Ronald Veldema (rveldema@cs.vu.nl)
  */
-public class UIDefaults extends Hashtable
+public class UIDefaults extends Hashtable<Object, Object>
 {
 
   /** Our ResourceBundles. */
@@ -95,10 +96,14 @@ public class UIDefaults extends Hashtable
     }
     public Object createValue(UIDefaults table)
     {
-      InputMap im = new InputMap ();
+      InputMapUIResource im = new InputMapUIResource ();
       for (int i = 0; 2*i+1 < bind.length; ++i)
         {
-          im.put (KeyStroke.getKeyStroke ((String) bind[2*i]),
+          Object curr = bind[2*i];
+          if (curr instanceof KeyStroke)
+            im.put((KeyStroke) curr, bind[2*i+1]);
+          else
+            im.put(KeyStroke.getKeyStroke((String) curr),
                   bind[2*i+1]);
         }
       return im;
@@ -667,7 +672,7 @@ public class UIDefaults extends Hashtable
    *
    * @return the UI class for <code>id</code>
    */
-  public Class getUIClass(String id, ClassLoader loader)
+  public Class<? extends ComponentUI> getUIClass(String id, ClassLoader loader)
   {
     String className = (String) get (id);
     if (className == null)
@@ -676,7 +681,7 @@ public class UIDefaults extends Hashtable
       {
         if (loader == null)
           loader = ClassLoader.getSystemClassLoader();
-        return loader.loadClass (className);
+        return (Class<? extends ComponentUI>) loader.loadClass (className);
       }
     catch (Exception e)
       {
@@ -693,7 +698,7 @@ public class UIDefaults extends Hashtable
    *
    * @return the UI class for <code>id</code>
    */
-  public Class getUIClass(String id)
+  public Class<? extends ComponentUI> getUIClass(String id)
   {
     return getUIClass (id, null);
   }
