@@ -376,7 +376,7 @@ public abstract class AbstractDocument implements Document, Serializable
    *
    * @return the properties of this <code>Document</code>
    */
-  public Dictionary getDocumentProperties()
+  public Dictionary<Object, Object> getDocumentProperties()
   {
     // FIXME: make me thread-safe
     if (properties == null)
@@ -541,6 +541,9 @@ public abstract class AbstractDocument implements Document, Serializable
     
     writeLock();
     UndoableEdit undo = content.insertString(offset, text);
+    if (undo != null)
+      event.addEdit(undo);
+
     insertUpdate(event, attributes);
     writeUnlock();
 
@@ -672,20 +675,8 @@ public abstract class AbstractDocument implements Document, Serializable
       new DefaultDocumentEvent(offset, length,
 			       DocumentEvent.EventType.REMOVE);
     
-    // Here we set up the parameters for an ElementChange, if one
-    // needs to be added to the DocumentEvent later
-    Element root = getDefaultRootElement();
-    int start = root.getElementIndex(offset);
-    int end = root.getElementIndex(offset + length);
-    
-    Element[] removed = new Element[end - start + 1];
-    for (int i = start; i <= end; i++)
-      removed[i - start] = root.getElement(i);
-    
     removeUpdate(event);
 
-    Element[] added = new Element[1];
-    added[0] = root.getElement(start);
     boolean shouldFire = content.getString(offset, length).length() != 0;
     
     writeLock();
@@ -694,17 +685,6 @@ public abstract class AbstractDocument implements Document, Serializable
     
     postRemoveUpdate(event);
     
-    GapContent.UndoRemove changes = null;
-    if (content instanceof GapContent)
-      changes = (GapContent.UndoRemove) temp;
-
-    if (changes != null && !(start == end))
-      {
-        // We need to add an ElementChange to our DocumentEvent
-        ElementEdit edit = new ElementEdit (root, start, removed, added);
-        event.addEdit(edit);
-      }
-    
     if (shouldFire)
       fireRemoveUpdate(event);
   }
@@ -849,7 +829,7 @@ public abstract class AbstractDocument implements Document, Serializable
    *
    * @param p the document properties to set
    */
-  public void setDocumentProperties(Dictionary p)
+  public void setDocumentProperties(Dictionary<Object, Object> p)
   {
     // FIXME: make me thread-safe
     properties = p;
@@ -1023,7 +1003,7 @@ public abstract class AbstractDocument implements Document, Serializable
      * @return the attributes of <code>old</code> minus the attributes in
      *         <code>attributes</code>
      */
-    AttributeSet removeAttributes(AttributeSet old, Enumeration names);
+    AttributeSet removeAttributes(AttributeSet old, Enumeration<?> names);
   }
 
   /**
@@ -1279,7 +1259,7 @@ public abstract class AbstractDocument implements Document, Serializable
      *
      * @param names the names of the attributes to be removed
      */
-    public void removeAttributes(Enumeration names)
+    public void removeAttributes(Enumeration<?> names)
     {
       attributes = getAttributeContext().removeAttributes(attributes, names);
     }
@@ -1349,7 +1329,14 @@ public abstract class AbstractDocument implements Document, Serializable
      */
     public Object getAttribute(Object key)
     {
-      return attributes.getAttribute(key);
+      Object result = attributes.getAttribute(key);
+      if (result == null && element_parent != null)
+        {
+          AttributeSet parentSet = element_parent.getAttributes();
+          if (parentSet != null)
+            result = parentSet.getAttribute(key);
+        }
+      return result;
     }
 
     /**
@@ -1367,7 +1354,7 @@ public abstract class AbstractDocument implements Document, Serializable
      *
      * @return the names of the attributes of this element
      */
-    public Enumeration getAttributeNames()
+    public Enumeration<?> getAttributeNames()
     {
       return attributes.getAttributeNames();
     }
@@ -1831,6 +1818,12 @@ public abstract class AbstractDocument implements Document, Serializable
      */
     Hashtable changes;
 
+    /**
+     * Indicates if this event has been modified or not. This is used to
+     * determine if this event is thrown.
+     */
+    boolean modified;
+
     /**
      * Creates a new <code>DefaultDocumentEvent</code>.
      *
@@ -1845,6 +1838,7 @@ public abstract class AbstractDocument implements Document, Serializable
       this.length = length;
       this.type = type;
       changes = new Hashtable();
+      modified = false;
     }
 
     /**
@@ -1859,6 +1853,7 @@ public abstract class AbstractDocument implements Document, Serializable
       // XXX - Fully qualify ElementChange to work around gcj bug #2499.
       if (edit instanceof DocumentEvent.ElementChange)
         {
+          modified = true;
           DocumentEvent.ElementChange elEdit =
             (DocumentEvent.ElementChange) edit;
           changes.put(elEdit.getElement(), elEdit);
@@ -1919,6 +1914,15 @@ public abstract class AbstractDocument implements Document, Serializable
       // XXX - Fully qualify ElementChange to work around gcj bug #2499.
       return (DocumentEvent.ElementChange) changes.get(elem);
     }
+    
+    /**
+     * Returns a String description of the change event.  This returns the
+     * toString method of the Vector of edits.
+     */
+    public String toString()
+    {
+      return edits.toString();
+    }
   }
   
   /**
@@ -2143,7 +2147,10 @@ public abstract class AbstractDocument implements Document, Serializable
      */
     public String getName()
     {
-      return ContentElementName;
+      String name = super.getName();
+      if (name == null)
+        name = ContentElementName;
+      return name;
     }
 
     /**
