@@ -168,7 +168,7 @@ public class StyleContext
       return attributes.getAttributeCount();
     }
 
-    public Enumeration getAttributeNames()
+    public Enumeration<?> getAttributeNames()
     {
       return attributes.getAttributeNames();
     }
@@ -195,7 +195,7 @@ public class StyleContext
       fireStateChanged();
     }
 
-    public void removeAttributes(Enumeration names)
+    public void removeAttributes(Enumeration<?> names)
     {
       attributes = StyleContext.this.removeAttributes(attributes, names);
       fireStateChanged();
@@ -323,7 +323,7 @@ public class StyleContext
       return attrs.length / 2;
     }
 
-    public Enumeration getAttributeNames()
+    public Enumeration<?> getAttributeNames()
     {      
       return new Enumeration() 
         {
@@ -362,9 +362,8 @@ public class StyleContext
 	
     public boolean isEqual(AttributeSet attr)
     {
-      return attr != null 
-        && attr.containsAttributes(this)
-        && this.containsAttributes(attr);
+      return getAttributeCount() == attr.getAttributeCount()
+             && this.containsAttributes(attr);
     }
 	
     public String toString()
@@ -479,7 +478,7 @@ public class StyleContext
    * Get the names of the style. The returned enumeration always
    * contains at least one member, the default style.
    */
-  public Enumeration getStyleNames()
+  public Enumeration<?> getStyleNames()
   {
     return styleTable.keys();
   }
